##############################################################################
#
# Copyright (c) 2019 Jens Vagelpohl and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
from bjoern import run


def serve_paste(app, global_conf, **kw):
    """ A handler for PasteDeploy-compatible runners.

    Sample .ini configuration:

     [server:main]
     use = egg:bjoern#main
     host = 127.0.0.1
     port = 8080
     reuse_port = True

    If no host is specified bjoern will bind to all IPv4 IPs (0.0.0.0)
    If no port is specified, bjoern will use a random port
    reuse_port defaults to False
    """
    # Convert the values from the .ini file to something bjoern can work with
    host = kw.get('host', '')
    port = int(kw.get('port', '0')) or False
    if not host and not port and kw.get('listen'):
        listen = kw.get('listen')
        if ':' in listen:
            host = listen.split(':')[0]
            port = int(listen.split(':')[1])
        else:
            host = ''
            port = int(listen)
    if kw.get('reuse_port', '').lower() in ('1', 'true', 'on'):
        reuse_port = True
    else:
        reuse_port = False

    run(app, host, port=port, reuse_port=reuse_port)
    return 0



##############################################################################
#
# Copyright (c) 2019 Jens Vagelpohl and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
import unittest


SERVER = {}


def fake_run(app, host, **kw):
    global SERVER
    SERVER.clear()

    SERVER['app'] = app
    SERVER['host'] = host
    SERVER.update(kw)


class BjoernInitializationTests(unittest.TestCase):

    def test_initialization(self):
        # Install fake server class first
        from dataflake.wsgi import bjoern
        old_impl = bjoern.run
        bjoern.run = fake_run

        from dataflake.wsgi.bjoern import serve_paste

        global SERVER

        # The defaults
        serve_paste(None, None)
        self.assertEqual(SERVER['host'], '')
        self.assertEqual(SERVER['port'], False)
        self.assertEqual(SERVER['reuse_port'], False)

        # Host and port set
        serve_paste(None, None, host='localhost', port='8888', reuse_port='On')
        self.assertEqual(SERVER['host'], 'localhost')
        self.assertEqual(SERVER['port'], 8888)
        self.assertEqual(SERVER['reuse_port'], True)

        # Listen without host
        serve_paste(None, None, listen='8898')
        self.assertEqual(SERVER['host'], '')
        self.assertEqual(SERVER['port'], 8898)

        # Listen with host
        serve_paste(None, None, listen='127.0.0.2:8999')
        self.assertEqual(SERVER['host'], '127.0.0.2')
        self.assertEqual(SERVER['port'], 8999)

        # Clean up fake server class
        bjoern.run = old_impl



##############################################################################
#
# Copyright (c) 2019 Jens Vagelpohl and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
import os
import sys
import unittest


class FakeMkwsgiinstance:

    def main(self):
        pass


class ConfigBuilderTests(unittest.TestCase):

    def test_argument_switch(self):
        # Patch a fake object for the mkwsgiinstance import
        from dataflake.wsgi.bjoern import configurator
        old_impl = configurator.mkwsgiinstance
        configurator.mkwsgiinstance = FakeMkwsgiinstance()

        # Save the "real" sys.argv
        old_args = sys.argv

        # Value passed in, no change
        sys.argv = [None, '-s', 'passed-in']
        configurator.mkzope()
        self.assertEqual(sys.argv, [None, '-s', 'passed-in'])

        # value passwd in, no change
        sys.argv = [None, '--skelsrc=passed-in']
        configurator.mkzope()
        self.assertEqual(sys.argv, [None, '--skelsrc=passed-in'])

        # Value not passed in, now we have a change
        skel_path = os.path.join(os.path.dirname(configurator.__file__),
                                 'skel')
        sys.argv = [None]
        configurator.mkzope()
        self.assertEqual(sys.argv, [None, '-s', skel_path])

        # clean up the patched module variable
        configurator.mkwsgiinstance = old_impl

        # Restore sys.argv
        sys.argv = old_args



##############################################################################
#
# Copyright (c) 2019 Jens Vagelpohl and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
import os
import sys

from Zope2.utilities import mkwsgiinstance


def mkzope():
    """ Create a WSGI configuration for Zope and bjoern """
    # We want to override the source of the skeleton files.
    # Make sure they are not already passed in.
    already_set = False

    for arg in sys.argv[1:]:
        if arg in ('-s', '--skelsrc') or arg.startswith('--skelsrc'):
            already_set = True
            break

    if not already_set:
        # Only go into action of the skeleton source has not been passed in.
        skelsrc = os.path.join(os.path.dirname(__file__), 'skel')
        sys.argv.extend(['-s', skelsrc])

    return mkwsgiinstance.main()



##############################################################################
#
# Copyright (c) 2019 Jens Vagelpohl and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
import os

from setuptools import find_packages
from setuptools import setup


NAME = 'dataflake.wsgi.bjoern'
URL = f'https://github.com/dataflake/{NAME}'


def read(*rnames):
    with open(os.path.join(os.path.dirname(__file__), *rnames)) as f:
        return f.read()


setup(
    name=NAME,
    version='2.0',
    url=URL,
    project_urls={
        'Documentation': 'https://dataflakewsgibjoern.readthedocs.io',
        'Issue Tracker': f'{URL}/issues',
        'Sources': URL,
    },
    license='ZPL 2.1',
    description='PasteDeploy entry point for the bjoern WSGI server',
    author='Jens Vagelpohl and Contributors',
    author_email='jens@dataflake.org',
    long_description=(read('README.rst') + '\n\n' + read('CHANGES.rst')),
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['dataflake', 'dataflake.wsgi'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Zope',
        'Framework :: Zope :: 5',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Zope Public License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
    ],
    python_requires='>=3.7',
    install_requires=[
        'setuptools',
        'bjoern',
        'paste',  # For the translogger logging filter
        'Zope >= 5',  # To avoid reinventing the skeleton creation
    ],
    extras_require={
        'docs': [
            'Sphinx',
            'sphinx_rtd_theme',
        ],
    },
    entry_points={
        'paste.server_runner': [
            'main=dataflake.wsgi.bjoern:serve_paste',
        ],
        'console_scripts': [
            'mkbjoerninstance=dataflake.wsgi.bjoern.configurator:mkzope',
        ],
    },
    include_package_data=True,
    zip_safe=False,
)
